<?php
/* Copyright (c) 2013, SWITCH */

class SessionModel implements \OAuth2\Storage\SessionInterface {

	private $db;

	public function __construct()
	{
		require_once 'lib/DB.php';
		$this->db = new DB();
	}

	/*
	 * @param $clientId
	 * @param $redirectUri
	 * @param $type
	 * @param $typeId the ID of the owner (for user type, the user ID)
	 * @param $authCode
	 * @param $accessToken
	 * @param $refreshToken
	 * @param $accessTokenExpire
	 * @param $stage
	 * @param $persistentId the AAI persistent ID of the Login
	 * @param $device A string identifying the mobile device
	 */
	public function createSession($clientId, $redirectUri, $type = 'user', $typeId = null, $authCode = null, $accessToken = null, $refreshToken = null, $accessTokenExpire = null, $stage = 'requested', $persistentId = null, $device = null)
	{
		if($persistentId === null){
			throw new Exception("Argument persistentId cannot be null");
		}
		$this->db->query('
			INSERT INTO oauth_sessions (
				client_id,
				redirect_uri,
				owner_type,
				owner_id,
				auth_code,
				access_token,
				refresh_token,
				access_token_expires,
				stage,
				first_requested,
				last_updated
			)
			VALUES (
				:clientId,
				:redirectUri,
				:type,
				:typeId,
				:authCode,
				:accessToken,
				:refreshToken,
				:accessTokenExpire,
				:stage,
				UNIX_TIMESTAMP(NOW()),
		UNIX_TIMESTAMP(NOW())
	)', array(
		':clientId' =>	$clientId,
		':redirectUri'	=>	$redirectUri,
		':type' =>	$type,
		':typeId'		=>	$typeId,
		':authCode' =>	$authCode,
		':accessToken'	=>	$accessToken,
		':refreshToken' =>	$refreshToken,
		':accessTokenExpire'		=>	$accessTokenExpire,
		':stage'		=>	$stage
	));

		$insertId = $this->db->getInsertId();

		$this->db->query('
			INSERT INTO aai_ids (
				sessionId,
				persistentId,
				device
			)
			VALUES ( 
				:sessionId,
				:persistentId,
				:device
			)', array(
				':sessionId' => $insertId,
				':persistentId' => $persistentId,
				':device' => $device
			));

		return $insertId;
	}

	public function updateSession($sessionId, $authCode = null, $accessToken = null, $refreshToken = null, $accessTokenExpire = null, $stage = 'requested') {
		$this->db->query('
			UPDATE oauth_sessions SET
			auth_code = :authCode,
			access_token = :accessToken,
			refresh_token = :refreshToken,
			access_token_expires = :accessTokenExpire,
			stage = :stage,
			last_updated = UNIX_TIMESTAMP(NOW())
			WHERE id = :sessionId',
			array(
				':authCode' =>	$authCode,
				':accessToken'	=>	$accessToken,
				':refreshToken' =>	$refreshToken,
				':accessTokenExpire'		=>	$accessTokenExpire,
				':stage'		=>	$stage,
				':sessionId'		=>	$sessionId
			));
	}

	public function deleteSession($clientId, $type, $typeId) {
		$this->db->query('
			DELETE FROM oauth_sessions WHERE
			client_id = :clientId AND
			owner_type = :type AND
			owner_id = :typeId',
			array(
				':clientId' =>	$clientId,
				':type'  =>  $type,
				':typeId' =>	$typeId
			));
	}

	public function revokeAccessToken($accessToken){
		$this->db->query('
			UPDATE oauth_sessions
			SET access_token_expires = 0
			WHERE access_token = ?',
			array($accessToken)
		);
	}


	public function validateAuthCode($clientId, $redirectUri, $authCode) {
		$result = $this->db->query('
			SELECT * FROM oauth_sessions WHERE
			client_id = :clientId AND
			redirect_uri = :redirectUri AND
			auth_code = :authCode',
			array(
				':clientId' =>	$clientId,
				':redirectUri'	=>	$redirectUri,
				':authCode' =>	$authCode
			));

		while ($row = $result->fetch())
		{
			return (array) $row;
		}

		return false;
	}

	public function validateAccessToken($accessToken) {
		$result = $this->db->query('SELECT id, owner_id, owner_type, client_id, access_token_expires FROM oauth_sessions WHERE access_token = :accessToken', array(':accessToken' => $accessToken));
		$row = $result->fetch();

		if ($row) {
			$expires = DateTime::createFromFormat('U',$row->access_token_expires);
			$now = new DateTime();
			if($expires < $now){ 
				//token expired
				return false;
			}
			return array(
				'id'		=>	$row->id,
				'owner_type' =>  $row->owner_type,
				'owner_id'	=>	$row->owner_id,
				'client_id' => $row->client_id
			);
		} else {
			return false;
		}
	}

	public function getAccessToken($sessionId)
	{
		$res = $this->db->query('SELECT access_token FROM oauth_sessions WHERE id = ?', array($sessionId));
		$row = $res->fetch();
		return $row->access_token;
	}

	public function validateRefreshToken($refreshToken, $clientId)
	{
		// Not needed for this demo
	}

	public function updateRefreshToken($sessionId, $newAccessToken, $newRefreshToken, $accessTokenExpires)
	{
		// Not needed for this demo
	}

	public function associateScope($sessionId, $scopeId) {
		$this->db->query('INSERT INTO oauth_session_scopes (session_id, scope_id) VALUE (:sessionId, :scopeId)', array(
			':sessionId'		=>	$sessionId,
			':scopeId'	=>	$scopeId
		));
	}

	public function getScopes($sessionId) {
		$result = $this->db->query('SELECT oauth_scopes.scope, oauth_scopes.name, oauth_scopes.description FROM oauth_session_scopes JOIN oauth_scopes ON oauth_session_scopes.scope_id = oauth_scopes.id WHERE session_id = :id', array(':id' => $sessionId));

		$scopes = array();

		while ($row = $result->fetch()) {
			$scopes[] = $row->scope;
		}

		return $scopes;
	}

	/**
	 * returns the persistentId for the given access token
	 */
	public function getPersistentId($accessToken){
		$result = $this->db->query('SELECT persistentId FROM aai_ids, oauth_sessions WHERE sessionId = id AND access_token = ?', array($accessToken));
		$row = $result->fetch();
		if($row){
			return $row->persistentId;
		}
		throw new Exception("Could not get persistent Id for access_token '$accessToken'");
	}

	/*
	 * returns all sessions owned by given unique id
	 */
	public function getSessions($uniqueId) {
		$result = $this->db->query('
			SELECT 
			oauth_sessions.id as sessionId,
			oauth_clients.prettyName as app,
			device,
			first_requested as requested,
			access_token_expires as expires
			FROM oauth_sessions, oauth_clients, aai_ids
			WHERE oauth_sessions.client_id = oauth_clients.id
			AND aai_ids.sessionId = oauth_sessions.id
			AND access_token_expires > UNIX_TIMESTAMP()
			AND owner_id = ?', 
			array($uniqueId)
		);
		$result->setFetchMode(PDO::FETCH_ASSOC);
		return $result->fetchAll();
	}
}
